#!/bin/bash

# Constants
TIMEOUT=36000
NB_RUNS=10

# Clean files
rm temp/*
rm results.txt
rm latex.txt

echo "" > temp/full_output.txt

# Learning tests on complete repressilator
#------------------------------------------

	if [ $1 = "1" ]; then

		echo "Starting experiment 1: LEARNING repressilator from complete input..."

		# Generate all state transitions for each 0.25 value of each variable in [0,1]
		./generator.exe input_benchmarks/repressilator.rules > temp/repressilator_complete.st 2>> temp/full_output.txt
		wait
	
		# Learn from the transitions with CLF1T
		./PRIDE.exe --algorithm=CONTINUUM temp/repressilator_complete.st > temp/repressilator_complete.rules 2>> temp/full_output.txt
		wait
	
		echo "Learned rules:"
		cat temp/repressilator_complete.rules
	fi
# Scalability tests on partial transitions of repressilator
#-----------------------------------------------------------

	if [ $1 = "2" ]; then

		echo "Starting experiment 2: SCALABILITY (number of transitions) with repressilator from partial input..."

		benchmarks=( 10 20 30 50 100 200 300 400 800 1000 10000 100000 1000000 10000000 100000000 )

		for i in "${benchmarks[@]}";
		do
			nb_transitions=$i
			sum_run_time="0.0"
			sum_memory=0
			
			echo -n "$nb_transitions & " >> latex.txt
		
			for ((run=0; run < NB_RUNS ; run++))
			do

				# Generate n state transitions for each 0.1 value of each variable in [0,1]
				./generator.exe input_benchmarks/repressilator.rules 0.1 -1 $nb_transitions > "temp/repressilator_"$nb_transitions"_transitions.st" 2>> temp/full_output.txt
				wait

				echo "Learning from $nb_transitions transitions:"

				# Learn from the transitions with CLF1T
				echo "Learning repressilator, $nb_transitions transitions:" >> temp/results.txt
				(TIMEFORMAT='%3R'; time ./PRIDE.exe --algorithm=CONTINUUM "temp/repressilator_"$nb_transitions"_transitions.st" 1> temp/repressilator_"$nb_transitions"_transitions.rules 2>> temp/full_output.txt) 2>> temp/results.txt
				wait
		
				memory=`tail -n 1 temp/full_output.txt`
		
				run_time=`tail -n 1 temp/results.txt`
				echo "Run time: "$run_time"s"
				echo "Memory: $memory rules"
				sum_run_time=$(echo "$sum_run_time + $run_time"|bc)
				sum_memory=$(echo "$sum_memory + $memory"|bc)
			done
		
			avg_run_time=$(echo "scale=3; $sum_run_time / $NB_RUNS"|bc)
			avg_memory=$(echo "$sum_memory / $NB_RUNS"|bc)

			echo "AVG Run time: "$avg_run_time"s"
			echo "AVG Memory: $avg_memory rules"
		
			#echo "Repressilator $nb_transitions transitions:" >> results.txt
			#echo "AVG Run time: "$avg_run_time"s" >> results.txt
			#echo "AVG Memory: $avg_memory rules" >> results.txt
			
			echo "$i,$avg_run_time,$avg_memory" >> results.txt
		done
	fi
	
# Scalability tests on partial mamalian cell
#-----------------------------------------------------------

	if [ $1 = "3" ]; then

		echo "Starting experiment 3: SCALABILITY (number of variables) with mammalian cell from partial input..."

		benchmarks=( "2" "3" "4" "5" "6" "7" "8" "9" )
		nb_transitions_tested=( 10 25 50 100 250 500 1000 )

		for nb_variables in "${benchmarks[@]}";
		do
			echo -n "\$$nb_variables\$ " >> latex.txt
			
			for nb_transitions in "${nb_transitions_tested[@]}";
			do
				#echo "$nb_transitions" >> latex.txt
		
				sum_run_time="0.0"
				sum_memory=0
		
				for ((run=0; run < NB_RUNS ; run++))
				do

					# Generate n state transitions for each 0.1 value of each variable in [0,1]
					./generator.exe input_benchmarks/mammalian_"$nb_variables".rules 0.1 -1 $nb_transitions > "temp/mammalian_"$nb_variables"_transitions.st" 2>> temp/full_output.txt
					wait

					echo "Learning from $nb_variables variables, $nb_transitions transitions:"

					# Learn from the transitions with CLF1T
					echo "Learning mammalian, $nb_variables variables:" >> temp/results.txt
					(TIMEFORMAT='%3R'; time timeout $TIMEOUT ./PRIDE.exe --algorithm=CONTINUUM "temp/mammalian_"$nb_variables"_transitions.st" 1> temp/mammalian_"$nb_variables"_transitions.rules 2>> temp/full_output.txt) 2>> temp/results.txt
					wait
		
					run_time=`tail -n 1 temp/results.txt`
					
					# Check Time out (cancel all next runs)
					if (( $(echo "$run_time >= $TIMEOUT" |bc -l) )); then
						echo "TIME OUT ($TIMEOUT seconds)"
						break
					fi
					
					memory=`tail -n 1 temp/full_output.txt`
					
					echo "Run time: "$run_time"s"
					echo "Memory: $memory rules"
					sum_run_time=$(echo "$sum_run_time + $run_time"|bc)
					sum_memory=$(echo "$sum_memory + $memory"|bc)
					
					
				done
				
				# Check Time out (cancel all next variables)
				if (( $(echo "$run_time >= $TIMEOUT" |bc -l) )); then
					echo -n " & T.O." >> latex.txt
					break
				fi
		
				avg_run_time=$(echo "scale=3; $sum_run_time / $NB_RUNS"|bc)
				avg_memory=$(echo "$sum_memory / $NB_RUNS"|bc)

				echo "AVG Run time: "$avg_run_time"s"
				echo "AVG Memory: $avg_memory rules"
		
				echo "Mamalian $nb_variables variables, $nb_transitions transitions:" >> results.txt
				echo "AVG Run time: "$avg_run_time"s" >> results.txt
				echo "AVG Memory: $avg_memory rules" >> results.txt
				
				echo -n " & \$ $avg_run_time \$ / \$ $avg_memory \$" >> latex.txt
				
			done
			
			echo "\\\\" >> latex.txt
		done
	fi
	
	echo "...finished."
